package extThopt;

import extThopt.*;
import java.util.*; 
import rg.corps.*; 
import rg.thopt.*; 
/**
* This class is an example of an external mixture for Thermoptim.
* Il allows you to model a food product such as orange juice or milk as proposed 
* CHOI, Y. and M.R. OKOS. 1986. Effects of temperature and composition on the thermal 
* properties of foods. In Food Engineering and Process Applications, 1:93-101. 
* M. LeMaguer and P. Jelen, eds. Elsevier Applied , Science Publishers, London.
*<p>This class may be modified under the CeCILL copyright

Copyright Renaud GICQUEL 2020  
renaud.gicquel@s4e2.com

Ce logiciel est un programme informatique servant à modéliser des systèmes énergétiques.   

Ce logiciel est régi par la licence CeCILL soumise au droit français et respectant les principes de diffusion des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les conditions de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA  sur le site "http://www.cecill.info".  

En contrepartie de l'accessibilité au code source et des droits de copie, de modification et de redistribution accordés par cette licence, il n'est offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons, seule une responsabilité restreinte pèse sur l'auteur du programme,  le titulaire des droits patrimoniaux et les concédants successifs.  

A cet égard  l'attention de l'utilisateur est attirée sur les risques associés au chargement,  à l'utilisation,  à la modification et/ou au développement et à la reproduction du logiciel par l'utilisateur étant  donné sa spécificité de logiciel libre, qui peut le rendre complexe à  manipuler et qui le réserve donc à des développeurs et des professionnels avertis possédant  des  connaissances  informatiques approfondies.  Les utilisateurs sont donc invités à charger  et  tester  l'adéquation  du logiciel à leurs besoins dans des conditions permettant d'assurer la sécurité de leurs systèmes et ou de leurs données et, plus généralement,  à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.   

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez  pris connaissance de la licence CeCILL, et que vous en avez accepté les termes. 

This software is a computer program whose purpose is to model energy systems.  

This software is governed by the CeCILL  license under French law and abiding by the rules of distribution of free software.  You can  use,  modify and/ or redistribute the software under the terms of the CeCILL license as circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".   

As a counterpart to the access to the source code and  rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty  and the software's author,  the holder of the economic rights,  and the successive licensors  have only  limited liability.   

In this respect, the user's attention is drawn to the risks associated with loading,  using,  modifying and/or developing or reproducing the software by the user in light of its specific status of free software, that may mean  that it is complicated to manipulate,  and  that  also therefore means  that it is reserved for developers  and  experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the software's suitability as regards their requirements in conditions enabling the security of their systems and/or  data to be ensured and,  more generally, to use and operate it in the  same conditions as regards security.   

The fact that you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.  
 * 
 * @author Renaud GICQUEL
*/

public abstract class FoodProduct extends ExtSubstance 
{
	private static java.util.ResourceBundle resInt_fr = java.util.ResourceBundle.getBundle("IntTh",Locale.getDefault());  
    
    Corps eau;
    boolean isCreated=false;
    double[] dryProductComp={0.59, 0.14, 9.85, 0.20, 0.41};
    double dryProductCompSum=11.17;
    String[] dryProductNames={"Protein","Fat","Carbohydrate","Fiber","Ash"};

    public FoodProduct(){//throws Exception{
        super();
        type=getType();
        M=18.01528;PC=221.2;TC=647.3;
        Tmini=273.15;Tmaxi=1073.15;
	    Pmaxi=1000.;
        chemForm="orange juice";
        typeCorps=6;//corps externe de type isMixture
  	    vMixtures= new Vector();
  	    setProductComp();
        dryProductCompSum=0;
        for(int i=0;i<5;i++)dryProductCompSum=dryProductCompSum+dryProductComp[i];

    }
    
    public String getType(){
        return getCompType();
    }
    
    /*
     * (to be overridden)
     * @see extThopt.ExtSubstance#getCompType()
     */
    public String getCompType(){
        return "food product";
    }
    /*
     * (to be overridden)
     */
    public void setProductComp(){
        dryProductComp[0]=0.59;
        dryProductComp[1]=0.14;
        dryProductComp[2]=9.85;
        dryProductComp[3]=0.20;
        dryProductComp[4]=0.41;
     }
    

    
	public double getSatPressure(double T, double x){
	    if(!isCreated){
            Object obj=Corps.createSubstance(resInt_fr.getString("eau"));
            eau=(Corps)obj;
            isCreated=true;
        }
	    double P=eau.getSatPressure(T-getDelay(x),0);
	    return P;
	}
	public double getSatTemperature(double P, double x){
	    if(!isCreated){
            Object obj=Corps.createSubstance(resInt_fr.getString("eau"));
            eau=(Corps)obj;
        }
	    double T=eau.getSatTemperature(P,0);
	    double delay=getDelay(x);
	    return T+delay;
	}
	
	double getDelay(double x){
/*	    if(x<0.062)return 1.;
	    if(x<0.11)return 2.;
	    if(x<0.145)return 3.;
	    if(x<0.176)return 4.;
	    if(x<0.203)return 5.;
	    else return 7.;*/
		 return 0.33*Math.exp(4.*x);
	}
    
	public double calcV (double T, double P,double x) {
		double t=T-273.15;
		double Xp=dryProductComp[0]*x/dryProductCompSum;
		double Xf=dryProductComp[1]*x/dryProductCompSum;
		double Xc=dryProductComp[2]*x/dryProductCompSum;
		double Xfi=dryProductComp[3]*x/dryProductCompSum;
		double Xa=dryProductComp[4]*x/dryProductCompSum;
		double Xw=1.-x;
		double Cw=1./1000.;
		double Cp = 1./(1.3299 *1.e3 - 5.1840  *1.e-1*t);
		double Cf = 1./(9.2559 *1.e2 - 4.1757 *1.e-1*t);
		double Cc = 1./(1.5991 *1.e3 - 3.1046 *1.e-1*t);
		double Cfi =1./( 1.3115 *1.e3 - 3.6589 *1.e-1*t);
		double Ca = 1./(2.4238 *1.e3 - 2.8063 *1.e-1*t);
		double C =Cw*Xw + Cp*Xp + Cf*Xf + Cc*Xc + Cfi*Xfi + Ca*Xa;		

		return C;
	}
	public double calcS (double T, double P,double x) {
		double t=T-273.15;
		double Xp=dryProductComp[0]*x/dryProductCompSum;
		double Xf=dryProductComp[1]*x/dryProductCompSum;
		double Xc=dryProductComp[2]*x/dryProductCompSum;
		double Xfi=dryProductComp[3]*x/dryProductCompSum;
		double Xa=dryProductComp[4]*x/dryProductCompSum;
		double Xw=1.-x;
		double Cw = 4.1762*Math.log(T) -9.0864 *1.e-5*t + 5.4731  *1.e-6*t*t/2 ;
		double Cp = 2.0082*Math.log(T) + 1.2089 *1.e-3*t - 1.3129 *1.e-6*t*t/2;
		double Cf = 1.9842*Math.log(T) + 1.4733 *1.e-3*t - 4.8008  *1.e-6*t*t/2;
		double Cc = 1.5488*Math.log(T) + 1.9625 *1.e-3*t - 5.9399  *1.e-6*t*t/2;
		double Cfi = 1.8459*Math.log(T) + 1.8306 *1.e-3*t - 4.6509 *1.e-6*t*t/2;
		double Ca = 1.0926*Math.log(T) + 1.8896 *1.e-3*t - 3.6817  *1.e-6*t*t/2;
		double C =Cw*Xw + Cp*Xp + Cf*Xf + Cc*Xc + Cfi*Xfi + Ca*Xa;
		return C-21;
	}
		public double calcXx (double T, double P,double x) {
	    return x;
	}

	public double calcH (double T, double P,double x) {
		double t=T-273.15;
		double Xp=dryProductComp[0]*x/dryProductCompSum;
		double Xf=dryProductComp[1]*x/dryProductCompSum;
		double Xc=dryProductComp[2]*x/dryProductCompSum;
		double Xfi=dryProductComp[3]*x/dryProductCompSum;
		double Xa=dryProductComp[4]*x/dryProductCompSum;
		double Xw=1.-x;
		double Cw = 4.1762*t -9.0864*1.e-5*t*t/2 + 5.4731 *1.e-6*t*t*t/3 ;
		double Cp = 2.0082*t + 1.2089 *1.e-3*t*t/2 - 1.3129 *1.e-6*t*t*t/3;
		double Cf = 1.9842*t + 1.4733 *1.e-3*t*t/2 - 4.8008  *1.e-6*t*t*t/3;
		double Cc = 1.5488*t + 1.9625 *1.e-3*t*t/2 - 5.9399  *1.e-6*t*t*t/3;
		double Cfi = 1.8459*t + 1.8306 *1.e-3*t*t/2 - 4.6509 *1.e-6*t*t*t/3;
		double Ca = 1.0926*t + 1.8896 *1.e-3*t*t/2 - 3.6817  *1.e-6*t*t*t/3;
		double C =Cw*Xw + Cp*Xp + Cf*Xf + Cc*Xc + Cfi*Xfi + Ca*Xa;
		return C;

	}



}